import numpy as np
import cv2 as cv
detector = cv.FaceDetectorYN.create(
   model='/usr/src/app/face_reg_v2/face_detection_yunet_2022mar.onnx',
   config='',
   input_size=[320, 320],
   score_threshold=0.73,
   nms_threshold=0.3,
   top_k=5000,
   backend_id=cv.dnn.DNN_BACKEND_OPENCV,
   target_id=cv.dnn.DNN_TARGET_CPU
)

def clamp_number(num,a,b,scale):
  v=max(min(num, max(a, b)), min(a, b))
  return round(v/scale)

def isInSide(top,right,bottom,left,landmark,scale):
    x,y = landmark
    x=x/scale
    y=y/scale
    return left <= x <= right and top <= y <= bottom

def isAllPartInBox(rect,face,scale):
    top,right,bottom,left = rect
    landmarks = face[4:14].astype(np.int32).reshape((5,2))
    for idx, landmark in enumerate(landmarks):
        isIn = isInSide(top,right,bottom,left,landmark,scale)
        if(isIn == False):
            return False
    return True
    
def isRectCross(rect1,rect2):
    top1,right1,bottom1,left1 = rect1
    top2,right2,bottom2,left2 = rect2
    if left1 >= right2 or right1 <= left2 or top1 >= bottom2 or bottom1 <= top2:
        return False
    return True

def getRect(face,w,h,scale,padding):
    top=clamp_number(face[1]-padding,0,h,scale)
    bottom=clamp_number(face[1]+face[3]+padding,0,h,scale)
    left=clamp_number(face[0]-padding,0,w,scale)
    right=clamp_number(face[0]+face[2]+padding,0,w,scale)
    return [top,right,bottom,left]

def getModelType(faceList,w,h,scale):
    crossTimes=0
    for idx, face in enumerate(faceList):
        for idx2, face2 in enumerate(faceList[idx+1:]):
            rect = getRect(face,w,h,scale,0)
            rect2 = getRect(face2,w,h,scale,0)
            if isRectCross(rect, rect2):
                crossTimes+=1
    if crossTimes>0:
        return "hog"
    return "cnn"
            
    
def cvLocationsToBox(faces,w,h,scale):
    result=[]
    if faces[1] is None:
        return {'model':"cnn",'list':result}

    model = getModelType(faces[1],w,h,scale)
    if model== "hog":
        return {'model':model,'list':result}

    for idx, face in enumerate(faces[1]):

        if face[2]+face[3] < 30:
            continue
        rect = getRect(face,w,h,scale,30)
        isInBox = isAllPartInBox(rect,face,scale)
        if not isInBox:
            continue

        top,right,bottom,left = rect
        result.append({'box':[top, right,bottom, left,float(face[-1])]})
    if len(result) == 0:
        return {'model':'hog','list':result}
    return {'model':model,'list':result}

    
def GetFaceLocations(imagPath):
    cvImg = cv.imread(imagPath)
    scale=1
    h, w, _ = cvImg.shape
    imgWidth = int(w*scale)
    imgHeight = int(h*scale)
    cvImg = cv.resize(cvImg, (imgWidth, imgHeight))
    detector.setInputSize([imgWidth, imgHeight])
    locations = detector.detect(cvImg)
    return cvLocationsToBox(locations,w,h,scale)
