import os
import socket
import dlib
import face_recognition
import json
import multiprocessing as mp
import numpy as np
from PIL import Image, ImageOps
from face_dect_cv import GetFaceLocations

IMAGE_PROXY_SIZE_WIDTH = 200
IMAGE_PROXY_SIZE_HEIGHT = 200
IMAGE_PROXY_SIZE = IMAGE_PROXY_SIZE_WIDTH * IMAGE_PROXY_SIZE_HEIGHT

server_address = '/tmp/mt_photos_face_reg.sock'

sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
sock.connect(server_address)

def Main ():
    status = 'run'
    imageList = []
    processes=1

    while status == "run" :
        msgData=GetMsg()
        if 'imageList' in msgData :
            imageList=msgData['imageList']
            processes=msgData['processes']

        elif 'status' in msgData :
            status=msgData['status']

        if len(imageList) > 0 and status == 'run':
            part=[]
            for i in range(0,processes):
                if len(imageList) > 0:
                    part.append(imageList[0])
                    del imageList[0]
            if len(part) > 0:
                pool = mp.Pool(processes=processes)
                res = pool.map(GetFaces, part)
                for resultData in res:
                    SendMsg(resultData)
        else :
            status="stop"

    sock.close()

def SendMsg(data):
    msg = json.dumps(data).encode('UTF-8')
    sock.send(msg)
    sock.send(b"\r\n")

def GetMsg():
    SendMsg({"type": "getMsg"})
    data = sock.recv(65536)
    dataStr=data.decode('UTF-8')
    msgData={}
    try :
        msgData=json.loads(dataStr)
    except ValueError :
        print('Decoding JSON has failed',dataStr)
    return msgData

def get_cnn_scale(width, height):
    scale = 1
    if width * height > IMAGE_PROXY_SIZE:
        if width > height:
            scale = IMAGE_PROXY_SIZE_WIDTH / width
        else :
            scale = IMAGE_PROXY_SIZE_HEIGHT / height
    return [round(width * scale), round(height * scale), scale]


def regPart(image, coord, i):
    # coord => [top,right,bottom,left,score]
    top=coord[0]
    right=coord[1]
    bottom=coord[2]
    left=coord[3]
    cropped = image[top:bottom,left:right]  # [top:bottom,left:right]

    [width, height, scale] = get_cnn_scale(right - left,bottom - top)
    if scale != 1 :
        cropped = dlib.resize_image(cropped, height, width)

    face_locations = face_recognition.face_locations(cropped,1,"cnn")
    if len(face_locations) == 0:
        return {'type':"empty"}
    # face locations order in css (top, right, bottom, left)
    box=[]
    for idx,val in enumerate(face_locations[0]) :
        if idx%2==0:
            box.append(round(val/scale+top))
        else:
            box.append(round(val/scale+left))
    face_encodings_list = face_recognition.face_encodings(cropped,face_locations,8,"large")
    return {'type':"cnn",'box':box,'face_encoding':face_encodings_list[0]}

def GetFacesWithHog(imagPath):
    result=[]
    image = load_image_file(imagPath)
    face_locations = face_recognition.face_locations(image)
    if len(face_locations) > 0:
        for inx,face_location in enumerate(face_locations):
            part_result=regPart(image,face_location,inx)
            if part_result['type'] == "cnn":
                result.append({'box':part_result['box'],'face_encoding':part_result['face_encoding'].tolist()})
    return result

def TryGetFaceLocations(imagPath):
    try:
        return GetFaceLocations(imagPath)
    except:
        return {'model':'none','list':[]}

def GetFaces(imagData):
    imagPath=imagData["path"]
    id=imagData["id"]
    result=[]
    if os.path.exists(imagPath) == False:
        return {'type':'face','id':id,'result':result}

    face_locations = TryGetFaceLocations(imagPath)
    model=face_locations['model']
    list=face_locations['list']
    print('model:',model)

    if model== "none":
        return {'type':'face','id':id,'result':result}
    if model== "hog":
        result = GetFacesWithHog(imagPath)
    else:
        if len(list) > 0:
            image = load_image_file(imagPath)
            for inx,face_location in enumerate(list):
                part_result=regPart(image,face_location['box'],inx)
                if part_result['type'] == "cnn":
                    result.append({'box':part_result['box'],'face_encoding':part_result['face_encoding'].tolist()})

    return {'type':'face','id':id,'result':result}

def load_image_file(file, mode='RGB'):
    try:
        im = Image.open(file)
        im = ImageOps.exif_transpose(im)
        if mode:
            im = im.convert(mode)
        return np.array(im)
    except:
        return face_recognition.load_image_file(file)

Main()

